/*!40101 SET @OLD_CHARACTER_SET_CLIENT=@@CHARACTER_SET_CLIENT */;
/*!40101 SET @OLD_CHARACTER_SET_RESULTS=@@CHARACTER_SET_RESULTS */;
/*!40101 SET @OLD_COLLATION_CONNECTION=@@COLLATION_CONNECTION */;
/*!40101 SET NAMES 'utf8' COLLATE 'utf8_general_ci' */;
/*!40103 SET @OLD_TIME_ZONE=@@TIME_ZONE */;
/*!40103 SET TIME_ZONE='+00:00' */;
/*!40014 SET @OLD_UNIQUE_CHECKS=@@UNIQUE_CHECKS, UNIQUE_CHECKS=0 */;
/*!40014 SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0 */;
/*!40101 SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='NO_AUTO_VALUE_ON_ZERO' */;
/*!40111 SET @OLD_SQL_NOTES=@@SQL_NOTES, SQL_NOTES=0 */;


-- 2017-09-07 15:04:49
-- Хранимая функция для определения последнего события среди связанных событий, содержащего действие Выписка
DELIMITER //
DROP FUNCTION IF EXISTS `getActionLeavedLastEventId`//
CREATE DEFINER = `root`@`localhost` FUNCTION `getActionLeavedLastEventId` (`aEventId` INT) RETURNS INT(11)
    COMMENT 'return id of last event in chain'
    DETERMINISTIC
    READS SQL DATA SQL SECURITY DEFINER

BEGIN
    DECLARE vCurrEventId INT;
    DECLARE vNextEventId INT DEFAULT NULL ;
    DECLARE vPath TEXT DEFAULT '';

    SET vCurrEventId = aEventId;
    LOOP
        IF FIND_IN_SET( vCurrEventId, vPath ) THEN
            RETURN aEventId;
        END IF ;
        SELECT IF( `NextEvent`.`deleted` , NULL , `NextEvent`.`id` ) INTO vNextEventId
        FROM `Event`
        LEFT JOIN `Event` AS `NextEvent` ON `NextEvent`.`prevEvent_id` = `Event`.`id`
        WHERE `Event`.`id` = vCurrEventId
        AND EXISTS(SELECT A.id
                            FROM Action AS A
                            INNER JOIN ActionType ON ActionType.id=A.actionType_id
                            WHERE A.event_id = NextEvent.id AND A.deleted=0 AND ActionType.deleted=0 AND ActionType.flatCode LIKE 'leaved%%')
        ORDER BY NextEvent.setDate DESC
        LIMIT 1 ;
        IF vNextEventId IS NULL THEN
            RETURN vCurrEventId;
        END IF ;
        SET vPath = CONCAT( vCurrEventId, ',', vPath ) ;
        SET vCurrEventId = vNextEventId;
    END LOOP;
END//


-- Хранимая функция для определения первого события среди связанных событий, содержащего действие Поступление
DELIMITER //
DROP FUNCTION IF EXISTS `getActionReceivedFirstEventId`//
CREATE DEFINER=`root`@`localhost` FUNCTION `getActionReceivedFirstEventId`(aEventId INT) RETURNS INT(11)
    READS SQL DATA
    DETERMINISTIC
    COMMENT 'return id of first event in chain'
BEGIN
    DECLARE vCurrEventId INT;
    DECLARE vPrevEventId INT  DEFAULT NULL;
    DECLARE vPath        TEXT DEFAULT '';

    SET vCurrEventId = aEventId;
    LOOP
        IF FIND_IN_SET(vCurrEventId, vPath) THEN
            RETURN aEventId;
        END IF;
        SELECT IF(`PrevEvent`.`deleted`, NULL, `PrevEvent`.`id`) INTO vPrevEventId
        FROM `Event`
        LEFT JOIN `Event` AS `PrevEvent` ON `PrevEvent`.`id` = `Event`.`prevEvent_id`
        WHERE `Event`.`id` = vCurrEventId
        AND EXISTS(SELECT A.id
                            FROM Action AS A
                            INNER JOIN ActionType ON ActionType.id=A.actionType_id
                            WHERE A.event_id = PrevEvent.id AND A.deleted=0 AND ActionType.deleted=0 AND ActionType.flatCode LIKE 'received%%')
        ORDER BY PrevEvent.setDate ASC
        LIMIT 1 ;
        IF vPrevEventId IS NULL THEN
            RETURN vCurrEventId;
        END IF;
        SET vPath = CONCAT(vCurrEventId, ',', vPath);
        SET vCurrEventId = vPrevEventId;
    END LOOP;
END//

DELIMITER ;



INSERT INTO `DatabaseVersion` (`base_version`, `updated`) VALUES ('up201709-02', NOW());

/*!40101 SET SQL_MODE=@OLD_SQL_MODE */;
/*!40014 SET FOREIGN_KEY_CHECKS=@OLD_FOREIGN_KEY_CHECKS */;
/*!40014 SET UNIQUE_CHECKS=@OLD_UNIQUE_CHECKS */;
/*!40101 SET CHARACTER_SET_CLIENT=@OLD_CHARACTER_SET_CLIENT */;
/*!40101 SET CHARACTER_SET_RESULTS=@OLD_CHARACTER_SET_RESULTS */;
/*!40101 SET COLLATION_CONNECTION=@OLD_COLLATION_CONNECTION */;
/*!40111 SET SQL_NOTES=@OLD_SQL_NOTES */;
