/*!40101 SET @OLD_CHARACTER_SET_CLIENT=@@CHARACTER_SET_CLIENT */;
/*!40101 SET @OLD_CHARACTER_SET_RESULTS=@@CHARACTER_SET_RESULTS */;
/*!40101 SET @OLD_COLLATION_CONNECTION=@@COLLATION_CONNECTION */;
/*!40101 SET NAMES 'utf8' COLLATE 'utf8_general_ci' */;
/*!40103 SET @OLD_TIME_ZONE=@@TIME_ZONE */;
/*!40103 SET TIME_ZONE='+00:00' */;
/*!40014 SET @OLD_UNIQUE_CHECKS=@@UNIQUE_CHECKS, UNIQUE_CHECKS=0 */;
/*!40014 SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0 */;
/*!40101 SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='NO_AUTO_VALUE_ON_ZERO' */;
/*!40111 SET @OLD_SQL_NOTES=@@SQL_NOTES, SQL_NOTES=0 */;


-- 2017-10-03
-- Утилизация ЛСиИМН

ALTER TABLE StockMotion
CHANGE `type`
`type` INT(11) DEFAULT '0' COMMENT '0-Накладная, 1-Инвентаризация, 2-финансовая переброска, 3-Производство, 4-Списание на пациента, 5-Возврат от пациента, 6-Резервирование на пациента, 7-Утилизация';

DELIMITER //
DROP PROCEDURE IF EXISTS generateTrans //
CREATE DEFINER=`root`@`localhost` PROCEDURE `generateTrans`(IN aMotionId       INT,
                               IN aMotionItemId   INT,
                               IN aNomenclatureId INT,
                               IN aBatch          VARCHAR(64) CHARSET utf8,
                               IN aShelfTime      DATE,
                               IN aOldFinanceId   INT,
                               IN aFinanceId      INT,
                               IN aOldQnt         DOUBLE,
                               IN aQnt            DOUBLE,
                               IN aOldSum         DOUBLE,
                               IN aSum            DOUBLE,
                               IN aIsOut          INT
                              )
    LANGUAGE SQL
    MODIFIES SQL DATA
    DETERMINISTIC
    COMMENT 'generate trans by motionItem'
BEGIN
    DECLARE vMotionType INT;
    DECLARE vDate       DATETIME;
    DECLARE vSupplierId INT;
    DECLARE vReceiverId INT;

    DECLARE vStockUnitId INT;
    DECLARE vUnitId INT;
    DECLARE vUnitRatio DOUBLE;
    DECLARE vQnt DOUBLE;
    DECLARE vOldQnt DOUBLE;

    SELECT `type`,     `date`, `supplier_id`, `receiver_id`
    INTO   vMotionType, vDate, vSupplierId,   vReceiverId
    FROM `StockMotion`
    WHERE `StockMotion`.`id` = aMotionId;

    SELECT `unit_id` INTO vUnitId FROM StockMotion_Item WHERE `id` = aMotionItemId;
    SELECT `defaultStockUnit_id` INTO vStockUnitId FROM `rbNomenclature` WHERE `id` = aNomenclatureId;

    IF vUnitId IS NOT NULL AND vStockUnitId IS NOT NULL THEN
        SELECT `ratio` INTO vUnitRatio FROM rbNomenclature_UnitRatio
        WHERE master_id = aNomenclatureId
        AND sourceUnit_id = vStockUnitId
        AND targetUnit_id = vUnitId
        LIMIT 1;
        IF vUnitRatio IS NOT NULL THEN
            SET vQnt = aQnt / vUnitRatio;
            IF vMotionType = 1 THEN -- 1-Инвентаризация
                SET vOldQnt = aOldQnt / vUnitRatio;
            END IF;
        ELSE
            SELECT `ratio` INTO vUnitRatio FROM rbNomenclature_UnitRatio
            WHERE master_id = aNomenclatureId
            AND sourceUnit_id = vUnitId
            AND targetUnit_id = vStockUnitId
            LIMIT 1;

            IF vUnitRatio IS NOT NULL THEN
                SET vQnt = aQnt * vUnitRatio;
                IF vMotionType = 1 THEN -- 1-Инвентаризация
                    SET vOldQnt = aOldQnt * vUnitRatio;
                END IF;
            ELSE
                SET vQnt = aQnt;
                IF vMotionType = 1 THEN -- 1-Инвентаризация
                    SET vOldQnt = aOldQnt;
                END IF;
            END IF;
        END IF;
    ELSE
        SET vQnt = aQnt;
    END IF;



    SELECT `type`,     `date`, `supplier_id`, `receiver_id`
    INTO   vMotionType, vDate, vSupplierId,   vReceiverId
    FROM `StockMotion`
    WHERE `StockMotion`.`id` = aMotionId;

    CASE vMotionType
        WHEN 0 THEN -- 0-Накладная
            INSERT
            INTO `StockTrans` (`stockMotionItem_id`, `date`, `qnt`, `sum`, `debOrgStructure_id`, `debNomenclature_id`, `debFinance_id`, `creOrgStructure_id`, `creNomenclature_id`, `creFinance_id`, `batch`, `shelfTime`)
            VALUES            ( aMotionItemId,        vDate,  vQnt,  aSum,  vReceiverId,          aNomenclatureId,      aFinanceId,      vSupplierId,          aNomenclatureId,      aFinanceId,      aBatch,  aShelfTime);
        WHEN 1 THEN -- 1-Инвентаризация
            INSERT
            INTO `StockTrans` (`stockMotionItem_id`, `date`, `qnt`,         `sum`,         `debOrgStructure_id`, `debNomenclature_id`, `debFinance_id`,  `creOrgStructure_id`, `creNomenclature_id`, `creFinance_id`, `batch`, `shelfTime`)
            VALUES            ( aMotionItemId,        vDate,  vQnt-vOldQnt,  aSum-aOldSum,  vSupplierId,          aNomenclatureId,      aFinanceId,      NULL,                  NULL,                NULL,             aBatch,  aShelfTime);
        WHEN 2 THEN -- 2-финансовая переброска
            INSERT
            INTO `StockTrans` (`stockMotionItem_id`, `date`, `qnt`, `sum`, `debOrgStructure_id`, `debNomenclature_id`, `debFinance_id`, `creOrgStructure_id`, `creNomenclature_id`, `creFinance_id`, `batch`, `shelfTime`)
            VALUES            ( aMotionItemId,        vDate,  vQnt,  aSum,  vSupplierId,          aNomenclatureId,      aFinanceId,      vSupplierId,          aNomenclatureId,      aOldFinanceId,   aBatch,  aShelfTime);
        WHEN 3 THEN -- 3-Производство
            IF aIsOut = 0 THEN
                INSERT
                INTO `StockTrans` (`stockMotionItem_id`, `date`, `qnt`, `sum`, `debOrgStructure_id`, `debNomenclature_id`, `debFinance_id`, `creOrgStructure_id`, `creNomenclature_id`, `creFinance_id`, `batch`, `shelfTime`)
                VALUES            ( aMotionItemId,        vDate,  vQnt,  aSum,  NULL,                NULL,                 NULL,            vSupplierId,          aNomenclatureId,      aFinanceId,      aBatch,  aShelfTime);
            ELSE
                INSERT
                INTO `StockTrans` (`stockMotionItem_id`, `date`, `qnt`, `sum`, `debOrgStructure_id`, `debNomenclature_id`, `debFinance_id`, `creOrgStructure_id`, `creNomenclature_id`, `creFinance_id`, `batch`, `shelfTime`)
                VALUES            ( aMotionItemId,        vDate,  vQnt,  aSum,  vSupplierId,          aNomenclatureId,      aFinanceId,      NULL,                NULL,                 NULL,            aBatch,   aShelfTime);
            END IF;
        WHEN 4 THEN -- 4-Списание на пациента
            INSERT
            INTO `StockTrans` (`stockMotionItem_id`, `date`, `qnt`, `sum`, `debNomenclature_id`, `debFinance_id`, `creOrgStructure_id`, `creNomenclature_id`, `creFinance_id`, `batch`, `shelfTime`)
            VALUES            ( aMotionItemId,        vDate,  vQnt,  aSum,  aNomenclatureId,      aFinanceId,      vSupplierId,          aNomenclatureId,      aFinanceId,      aBatch,  aShelfTime);
        WHEN 5 THEN -- 5-Возврат от пациента
            INSERT
            INTO `StockTrans` (`stockMotionItem_id`, `date`, `qnt`, `sum`, `debNomenclature_id`, `debFinance_id`, `debOrgStructure_id`, `creNomenclature_id`, `creFinance_id`, `batch`, `shelfTime`)
            VALUES            ( aMotionItemId,        vDate,  vQnt,  aSum,  aNomenclatureId,      aFinanceId,      vReceiverId,          aNomenclatureId,      aFinanceId,      aBatch,  aShelfTime);
        WHEN 6 THEN -- 6-резервирование на пациента
            INSERT
            INTO `StockTrans` (`stockMotionItem_id`, `date`, `qnt`, `sum`, `debNomenclature_id`, `debFinance_id`, `creOrgStructure_id`, `creNomenclature_id`, `creFinance_id`, `batch`, `shelfTime`)
            VALUES            ( aMotionItemId,        vDate,  vQnt,  aSum,  aNomenclatureId,      aFinanceId,      vSupplierId,          aNomenclatureId,      aFinanceId,      aBatch,  aShelfTime);
        WHEN 7 THEN -- 7-Утилизация
            INSERT
            INTO `StockTrans` (`stockMotionItem_id`, `date`, `qnt`, `sum`, `debNomenclature_id`, `debFinance_id`, `creOrgStructure_id`, `creNomenclature_id`, `creFinance_id`, `batch`, `shelfTime`)
            VALUES            ( aMotionItemId,        vDate,  vQnt,  aSum,  aNomenclatureId,      aFinanceId,      vSupplierId,          aNomenclatureId,      aFinanceId,      aBatch,  aShelfTime);
    END CASE;
END//
DELIMITER ;

INSERT INTO `DatabaseVersion` (`base_version`, `updated`) VALUES ('up201709-12', NOW());

/*!40101 SET SQL_MODE=@OLD_SQL_MODE */;
/*!40014 SET FOREIGN_KEY_CHECKS=@OLD_FOREIGN_KEY_CHECKS */;
/*!40014 SET UNIQUE_CHECKS=@OLD_UNIQUE_CHECKS */;
/*!40101 SET CHARACTER_SET_CLIENT=@OLD_CHARACTER_SET_CLIENT */;
/*!40101 SET CHARACTER_SET_RESULTS=@OLD_CHARACTER_SET_RESULTS */;
/*!40101 SET COLLATION_CONNECTION=@OLD_COLLATION_CONNECTION */;
/*!40111 SET SQL_NOTES=@OLD_SQL_NOTES */;
