/*!40101 SET @OLD_CHARACTER_SET_CLIENT=@@CHARACTER_SET_CLIENT */;
/*!40101 SET @OLD_CHARACTER_SET_RESULTS=@@CHARACTER_SET_RESULTS */;
/*!40101 SET @OLD_COLLATION_CONNECTION=@@COLLATION_CONNECTION */;
/*!40101 SET NAMES 'utf8' COLLATE 'utf8_general_ci' */;
/*!40103 SET @OLD_TIME_ZONE=@@TIME_ZONE */;
/*!40103 SET TIME_ZONE='+00:00' */;
/*!40014 SET @OLD_UNIQUE_CHECKS=@@UNIQUE_CHECKS, UNIQUE_CHECKS=0 */;
/*!40014 SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0 */;
/*!40101 SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='NO_AUTO_VALUE_ON_ZERO' */;
/*!40111 SET @OLD_SQL_NOTES=@@SQL_NOTES, SQL_NOTES=0 */;


-- 2018-02-01
-- 0009422: Наблюдаемый контингент, индикация с условием Действие

DROP function IF EXISTS `checkClientContingentTypeActionType`;

DELIMITER $$

CREATE DEFINER=`root`@`localhost` FUNCTION `checkClientContingentTypeActionType`(
    aClientId INT,
    aContingentTypeId INT) RETURNS tinyint(1)
    READS SQL DATA
    DETERMINISTIC
    COMMENT 'Return 0,1,2,3 by corresponds to rbContingentType_ActionType'
BEGIN

    DECLARE vResult TINYINT;
    DECLARE vResultActionId INT;
    DECLARE vResultActionEndDate DATE;

    DECLARE vControlPeriod TINYINT(1);
    DECLARE vControlBegDate DATE;
    DECLARE vControlEndDate DATE;
    DECLARE vActionTypeId INT;
    DECLARE vFreePeriod INT;

    DECLARE vContingentTypeActionTypeCursor CURSOR FOR SELECT `actionType_id`, `controlPeriod`, `freePeriod` FROM rbContingentType_ActionType
    WHERE `master_id` = aContingentTypeId;
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET vResult = NULL;

    SET vResult = 0;

    OPEN vContingentTypeActionTypeCursor;

    contingentTypeActionTypeLoop : LOOP
    FETCH vContingentTypeActionTypeCursor INTO vActionTypeId, vControlPeriod, vFreePeriod;

    IF ISNULL(vResult) THEN
      SET vResult = 0;
      LEAVE contingentTypeActionTypeLoop;
        END IF;

    CASE vControlPeriod
        WHEN 0 THEN SET vControlEndDate = DATE_ADD(DATE_ADD(CURDATE(), INTERVAL 1 YEAR), INTERVAL -DAYOFYEAR(DATE_ADD(CURDATE(), INTERVAL 1 YEAR)) DAY);
        SET vControlBegDate = DATE_ADD(vControlEndDate, INTERVAL -(DAYOFYEAR(vControlEndDate)-1) DAY);
        WHEN 1 THEN SET vControlBegDate = DATE_ADD(CURDATE(), INTERVAL -(vFreePeriod-1) DAY); SET vControlEndDate = CURDATE();
        WHEN 2 THEN SET vControlBegDate = DATE_ADD(DATE_ADD(CURDATE(), INTERVAL 1 DAY), INTERVAL -(vFreePeriod) WEEK); SET vControlEndDate = CURDATE();
        WHEN 3 THEN SET vControlBegDate = DATE_ADD(DATE_ADD(CURDATE(), INTERVAL 1 DAY), INTERVAL -(vFreePeriod) MONTH); SET vControlEndDate = CURDATE();
        WHEN 4 THEN SET vControlBegDate = DATE_ADD(DATE_ADD(CURDATE(), INTERVAL 1 DAY), INTERVAL -(vFreePeriod) YEAR); SET vControlEndDate = CURDATE();
        ELSE ITERATE contingentTypeActionTypeLoop;
    END CASE;


    IF ISNULL(vControlBegDate) THEN
      SELECT Action.`id`, Action.`endDate`
      FROM `Action` INNER JOIN Event ON (Action.event_id = Event.id  AND Event.`client_id`= aClientId AND Event.deleted = 0)
      WHERE Action.`actionType_id`=vActionTypeId AND Action.deleted = 0 AND DATE(Action.`begDate`) <= vControlEndDate
      ORDER BY Action.begDate DESC
      LIMIT 1 INTO vResultActionId, vResultActionEndDate;
    ELSE
      SELECT Action.`id`, Action.`endDate`
      FROM Action INNER JOIN `Event` ON (Action.event_id = Event.id AND Event.`client_id`= aClientId AND Event.deleted = 0)
      WHERE Action.`actionType_id`=vActionTypeId AND Action.deleted = 0 AND IF(Action.`begDate` is not NULL, DATE(Action.`begDate`) >= vControlBegDate AND DATE(Action.`begDate`) <= vControlEndDate, DATE(Action.`endDate`) >= vControlBegDate AND DATE(Action.`endDate`) <= vControlEndDate)
      ORDER BY Action.begDate DESC
      LIMIT 1 INTO vResultActionId, vResultActionEndDate;
    END IF;

    IF vResultActionId THEN
        IF ISNULL(vResultActionEndDate) THEN
        SET vResult = 1;
        ELSE
        SET vResult = 2;
        END IF;
    ELSE
      set vResult = 0;
    END IF;

    IF vResult > 0 THEN
        LEAVE contingentTypeActionTypeLoop;
        END IF;


    END LOOP contingentTypeActionTypeLoop;

    CLOSE vContingentTypeActionTypeCursor;

    RETURN vResult;
END$$

DELIMITER ;
  
INSERT INTO `DatabaseVersion` (`base_version`, `updated`) VALUES ('up201801-07', NOW());

/*!40101 SET SQL_MODE=@OLD_SQL_MODE */;
/*!40014 SET FOREIGN_KEY_CHECKS=@OLD_FOREIGN_KEY_CHECKS */;
/*!40014 SET UNIQUE_CHECKS=@OLD_UNIQUE_CHECKS */;
/*!40101 SET CHARACTER_SET_CLIENT=@OLD_CHARACTER_SET_CLIENT */;
/*!40101 SET CHARACTER_SET_RESULTS=@OLD_CHARACTER_SET_RESULTS */;
/*!40101 SET COLLATION_CONNECTION=@OLD_COLLATION_CONNECTION */;
/*!40111 SET SQL_NOTES=@OLD_SQL_NOTES */;
